import os
import re

from .. import helper
from ..ext_exception import ExtException
from ..metadata_types import MetaDataTypes


class MetaObject:
    version = '803'
    ext_code = {'obj': 0}
    re_meta_data_obj = re.compile(r'^[^.]+\.json$')
    directive_1c_uncomment = re.compile('(?P<n>\\n)(?P<d>[#|&])')
    directive_1c_comment = re.compile('(?P<n>\\n)(?P<c>// v8unpack )(?P<d>[#|&])')

    def __init__(self):
        self.header = {}
        self.code = {}

    @classmethod
    def get_decode_header(cls, header_data):
        return header_data[0][1][1]

    def set_header_data(self, header_data):
        _form_header = self.get_decode_header(header_data)
        helper.decode_header(self.header, _form_header)
        self.header['data'] = header_data

    def decode_includes(self, src_dir, dest_dir, dest_path, header_data):
        tasks = []
        includes = self.get_decode_includes(header_data)
        for include in includes:
            try:
                count_include_types = int(include[2])
            except IndexError:
                raise ExtException(msg='Include types not found', detail=self.__class__.__name__)
            for i in range(count_include_types):
                _metadata = include[i + 3]
                _count_obj = int(_metadata[1])
                _metadata_type_uuid = _metadata[0]
                if not _count_obj:
                    continue
                try:
                    metadata_type = MetaDataTypes(_metadata_type_uuid)
                except ValueError:
                    # data = helper.json_read(src_dir, f'{_metadata[2]}.json')  # чтобы посмотреть что это
                    # continue

                    if not isinstance(_metadata[2], str):  # вложенный объект
                        continue
                    msg = f'У {self.__class__.__name__} {self.header["name"]} неизвестный тип вложенных метаданных: {_metadata_type_uuid} лежит в файле {_metadata[2]}'
                    raise Exception(msg)
                new_dest_path = os.path.join(dest_path, metadata_type.name)
                for j in range(_count_obj):
                    obj_uuid = _metadata[j + 2]
                    if isinstance(obj_uuid, str):
                        if j == 0:
                            os.mkdir(os.path.join(dest_dir, new_dest_path))

                        tasks.append([metadata_type.name, [src_dir, obj_uuid, dest_dir, new_dest_path, self.version]])
                    elif isinstance(obj_uuid, list):
                        if not metadata_type:
                            continue
                        try:
                            handler = helper.get_class_metadata_object(metadata_type.name)
                        except Exception as err:
                            continue
                        if j == 0:
                            os.mkdir(os.path.join(dest_dir, new_dest_path))
                        handler.decode_local_include(self, obj_uuid, src_dir, dest_dir, new_dest_path, self.version)

        return tasks

    @classmethod
    def get_decode_includes(cls, header_data: list) -> list:
        raise Exception(f'Не метаданных {cls.__name__} не указана ссылка на includes')

    def encode_includes(self, src_dir, dest_dir):
        tasks = []
        includes = []
        entries = sorted(os.listdir(src_dir))
        for entry in entries:
            src_entry_path = os.path.join(src_dir, entry)
            if os.path.isdir(src_entry_path):
                try:
                    MetaDataTypes[entry]
                except KeyError:
                    raise Exception(f'Обнаружен не поддерживаемый тип метаданных или некорректно описанная область {entry} ')
                includes.append(entry)

        for include in includes:
            _handler = helper.get_class_metadata_object(include)
            _handler.encode_get_include_obj(os.path.join(src_dir, include), dest_dir, _handler.get_obj_name(), tasks,
                                            self.version)
        return tasks

    @classmethod
    def encode_get_include_obj(cls, src_dir, dest_dir, include, tasks, version):
        """
        возвращает список задач на парсинг объектов этого типа
        """
        entries = os.listdir(src_dir)
        for entry in entries:
            if cls.re_meta_data_obj.fullmatch(entry):
                tasks.append([include, [src_dir, entry[:-5], dest_dir, version]])

    @classmethod
    def encode_get_include_obj_from_named_folder(cls, src_dir, dest_dir, include, tasks, version):
        """
        возвращает список задач на парсинг объектов этого типа
        """
        entries = os.listdir(src_dir)
        for entry in entries:
            if os.path.isdir(os.path.join(src_dir, entry)):
                new_src_dir = os.path.join(src_dir, entry)
                tasks.append([include, [new_src_dir, include, dest_dir, version]])

    def encode_version(self):
        return self.header['version']

    @classmethod
    def get_class_name_without_version(cls):
        if cls.__name__.endswith(cls.version):
            return cls.__name__[:len(cls.version) * -1]
        return cls.__name__

    def read_raw_code(self, src_dir, file_name, encoding='utf-8'):
        code = helper.txt_read(src_dir, file_name, encoding=encoding)
        if code:
            # if self.version in ['801', '802']:  # убираем комментрии у директив
            code = self.directive_1c_comment.sub('\g<n>\g<d>', code)
        return code

    def write_raw_code(self, code, dest_dir, filename, encoding='uft-8'):
        if code is not None:
            if self.version in ['801', '802']:  # комментируем директивы
                code = self.directive_1c_uncomment.sub('\g<n>// v8unpack \g<d>', code)
            helper.txt_write(code, dest_dir, filename, encoding=encoding)

    def decode_code(self, src_dir):
        for code_name in self.ext_code:
            _obj_code_dir = f'{os.path.join(src_dir, self.header["uuid"])}.{self.ext_code[code_name]}'
            if os.path.isdir(_obj_code_dir):
                self.header[f'code_info_{code_name}'] = helper.json_read(_obj_code_dir, 'info.json')
                self.code[code_name] = self.read_raw_code(_obj_code_dir, 'text.txt')

                encoding = helper.detect_by_bom(os.path.join(_obj_code_dir, 'text.txt'), 'utf-8')
                self.header[f'code_encoding_{code_name}'] = encoding  # можно безболезненно поменять на utf-8-sig

    def write_decode_code(self, dest_dir, file_name):
        for code_name in self.code:
            if self.code[code_name]:
                helper.txt_write(self.code[code_name], dest_dir, f'{file_name}.{code_name}.1c')

    def encode_code(self, src_dir, file_name):
        for code_name in self.ext_code:
            if self.header.get(f'code_info_{code_name}'):
                try:
                    self.code[code_name] = helper.txt_read(src_dir, f'{file_name}.{code_name}.1c')
                except FileNotFoundError:
                    self.code[code_name] = ''

    def write_encode_code(self, dest_dir):
        for code_name in self.code:
            _code_dir = f'{os.path.join(dest_dir, self.header["uuid"])}.{self.ext_code[code_name]}'
            os.makedirs(_code_dir)
            helper.json_write(self.header[f'code_info_{code_name}'], _code_dir, 'info.json')
            self.write_raw_code(self.code[code_name], _code_dir, 'text.txt',
                                encoding=self.header.get(f'code_encoding_{code_name}', 'utf-8'))

    def set_product_version(self, product_version):
        pass

    def set_product_comment(self, product_version):
        header = self.get_decode_header(self.header['data'])
        header[4] = helper.str_encode(helper.str_decode(header[4]) + product_version)

    def set_product_info(self, src_dir, file_name):
        product_version = ''
        try:
            product_version = helper.txt_read(src_dir, 'version.txt', encoding='utf-8')
            self.set_product_version(product_version)
        except FileNotFoundError:
            pass
        if file_name:
            product_info = f'#{file_name}:{product_version}'
            self.set_product_comment(product_info)



import os

from . import helper
from .ext_exception import ExtException
from .index import get as get_from_index


class CodeOrganizer:
    def __init__(self):
        self.code_areas = None
        self.data = None

    @classmethod
    def unpack_mp(cls, params):
        return cls.unpack(*params)

    @classmethod
    def unpack(cls, src_dir, path, file_name, dest_dir, index):
        self = cls()
        self.code_areas = {'root': dict(data='')}
        _path = ['']
        _include_path = ['root']
        with open(os.path.join(src_dir, path, file_name), 'r', encoding='utf-8-sig') as file:
            line = file.readline()
            while line:
                try:
                    _line = line.strip()
                    if _line and _line[0] == '#':
                        if _line.startswith('#Область'):
                            if _line.startswith('#Область include'):
                                key = _line[17:].strip()
                                # if key in self.code_areas:
                                #     raise Exception(
                                #         f'В {path}{file_name} ссылка на один и тот же файл у разных областей {key}')
                                self.code_areas[_include_path[-1]]['data'] += line
                                self.code_areas[key] = dict(data='')
                                _include_path.append(key)
                                _path.append(key)
                            else:
                                self.code_areas[_include_path[-1]]['data'] += line
                                _path.append('')

                        elif _line.startswith('#КонецОбласти'):
                            if _path[-1]:  # кончилась include область
                                _include_path.pop()
                            _path.pop()
                            self.code_areas[_include_path[-1]]['data'] += line
                        else:
                            self.code_areas[_include_path[-1]]['data'] += line
                    else:
                        self.code_areas[_include_path[-1]]['data'] += line
                    line = file.readline()
                except Exception as err:
                    raise ExtException(parent=err, detail=f'in file {file_name} line {line}') from err
        return self.code_areas

    @classmethod
    def pack_mp(cls, params):
        return cls.pack(*params)

    @classmethod
    def pack(cls, src_dir, src_path, src_file_name, dest_dir, dest_path, dest_file_name, index_code_areas, descent,
             pack_get_descent_filename):
        data = cls.pack_file(src_dir, src_path, src_file_name, index_code_areas, descent, pack_get_descent_filename)
        helper.txt_write(data, os.path.join(dest_dir, dest_path), dest_file_name)

    @classmethod
    def pack_file(cls, src_dir, path, file_name, index_code_areas, descent, pack_get_descent_filename):
        try:
            data = ''
            with open(os.path.join(src_dir, path, file_name), 'r', encoding='utf-8') as file:
                line = file.readline()
                while line:
                    data += line
                    _line = line.strip()
                    if _line and _line[0] == '#':
                        if _line.startswith('#Область include'):
                            include_path = _line[17:].strip()
                            _path, _file_name = cls.parse_include_path(include_path, path, file_name, index_code_areas,
                                                                       descent)
                            _src_abs_path = os.path.abspath(os.path.join(src_dir, _path))
                            if _src_abs_path.startswith(src_dir):
                                _path, _file_name = pack_get_descent_filename(_src_abs_path, _file_name, descent)
                            data += cls.pack_file(src_dir, _path, _file_name, index_code_areas, descent,
                                                  pack_get_descent_filename)
                    line = file.readline()
                return data
        except Exception as err:
            raise ExtException(parent=err, action=f'{cls.__name__}.pack_file', detail=f'{path} {file_name}')

    @staticmethod
    def parse_include_path(include_path, path, file_name, index_code_areas, descent):
        if index_code_areas and include_path in index_code_areas:
            include_path = index_code_areas[include_path]
        tmp = include_path.split('_')
        size_tmp = len(tmp)
        if size_tmp == 0:
            raise Exception(f'{path} {file_name} в include не указан путь')
        _file_name = f'{tmp[-1]}.1c'
        _path = '..'  # include не должен лежать внутри папки с исходниками
        if descent:
            _path = os.path.join(_path, '..')
        if size_tmp > 1:
            tmp = ['..' if elem == '' else elem for elem in tmp[:-1]]
            _path = os.path.join(_path, *tmp)
        return _path, _file_name

    @staticmethod
    def get_dest_path(dest_dir: str, path: str, file_name: str, index: dict, descent: int):
        if index:
            try:
                _res = get_from_index(index, path, file_name)
            except KeyError:
                _res = None

            if _res:
                _path = os.path.dirname(_res)
                _file = os.path.basename(_res)
                _path = os.path.join(
                    '..',
                    '' if descent is None else '..',  # в режиме с descent корень находится на уровень выше
                    _path
                )

                try:
                    os.makedirs(os.path.join(dest_dir, _path), exist_ok=True)
                except FileExistsError:
                    pass
                return _path, _file

        return path, file_name



from ..MetaDataObject.core.Simple import SimpleNameFolder
from ..MetaDataObject.versions.DocumentForm803 import DocumentForm803


class InformationRegisterForm(SimpleNameFolder):
    versions = {
        '803': DocumentForm803
    }



from .Form803 import Form803


class CommonForm803(Form803):
    @classmethod
    def get_decode_obj_header(cls, header):
        return header[0][1]

    def decode_data(self, src_dir, uuid):
        super(CommonForm803, self).decode_data(src_dir, uuid)
        _header_obj = self.get_decode_obj_header(self.header['data'])
        self.header['Пояснение'] = _header_obj[3]
        self.header['Использовать стандартные команды'] = _header_obj[4]

    def encode_header(self):
        return [[
            "1",
            [
                "4",
                self.encode_header_title(),
                self.header['Расширенное представление'],
                self.header['Пояснение'],
                self.header['Использовать стандартные команды']
            ],
            "0"
        ]]



from .. import helper
from ..MetaDataObject.core.Container import Container


class Subsystem(Container):
    pass

    def decode_object(self, src_dir, file_name, dest_dir, dest_path, version, header_data):
        super(Subsystem, self).decode_object(src_dir, file_name, dest_dir, dest_path, version, header_data)
        try:
            self.header['info'] = helper.json_read(src_dir, f'{self.header["uuid"]}.1.json')
        except FileNotFoundError:
            self.header['info'] = None

    def write_encode_object(self, dest_dir):
        super(Subsystem, self).write_encode_object(dest_dir)
        if self.header['info']:
            helper.json_write(self.header['info'], dest_dir, f'{self.header["uuid"]}.1.json')



from .. import __version__
from .. import helper
from ..MetaObject import MetaObject


class ExternalDataProcessor(MetaObject):

    def __init__(self):
        super(ExternalDataProcessor, self).__init__()
        self.data = None

    @classmethod
    def decode(cls, src_dir, dest_dir, *, version=None):
        self = cls()
        self.header = {}
        self.data = {}
        root = helper.json_read(src_dir, 'root.json')
        self.header["file_uuid"] = root[0][1]
        _header_data = helper.json_read(src_dir, f'{self.header["file_uuid"]}.json')
        self.set_header_data(_header_data)

        root = helper.json_read(src_dir, 'root.json')
        self.header['v8unpack'] = __version__
        self.header['file_uuid'] = root[0][1]
        self.data['version'] = helper.json_read(src_dir, 'version.json')
        self.header['versions'] = helper.json_read(src_dir, 'versions.json')
        self.data['copyinfo'] = helper.json_read(src_dir, 'copyinfo.json')

        try:
            form1 = helper.json_read(src_dir, f'{self.header["uuid"]}.1.json')
        except FileNotFoundError:
            form1 = None

        self.data['form1'] = form1

        self.decode_code(src_dir)
        pass
        _file_name = self.get_class_name_without_version()
        helper.json_write(self.header, dest_dir, f'{_file_name}.json')
        helper.json_write(self.data, dest_dir, f'{_file_name}.data{self.version}.json')
        self.write_decode_code(dest_dir, 'ExternalDataProcessor')

        tasks = self.decode_includes(src_dir, dest_dir, '', self.header['data'])
        return tasks
        # helper.run_in_pool(self.decode_include, tasks, pool)
        pass

    @classmethod
    def get_decode_includes(cls, header_data):
        return [header_data[0][3][1]]

    @classmethod
    def get_decode_header(cls, header_data):
        return header_data[0][3][1][1][3][1]

    @classmethod
    def encode(cls, src_dir, dest_dir, *, version=None, release=None, file_name=None):
        self = cls()
        helper.clear_dir(dest_dir)
        _file_name = self.get_class_name_without_version()
        self.header = helper.json_read(src_dir, f'{_file_name}.json')
        helper.check_version(__version__, self.header.get('v8unpack', ''))
        try:
            self.data = helper.json_read(src_dir, f'{_file_name}.data{self.version}.json')
        except FileNotFoundError:
            self.data = self.encode_empty_data()

        self.set_product_info(src_dir, file_name)

        helper.json_write(self.encode_root(), dest_dir, 'root.json')
        helper.json_write(self.encode_version(), dest_dir, 'version.json')
        helper.json_write(self.header['versions'], dest_dir, 'versions.json')
        helper.json_write(self.data['copyinfo'], dest_dir, 'copyinfo.json')
        helper.json_write(self.header['data'], dest_dir, f'{self.header["file_uuid"]}.json')
        if self.data.get('form1'):
            helper.json_write(self.data['form1'], dest_dir, f'{self.header["uuid"]}.1.json')
        self.encode_code(src_dir, 'ExternalDataProcessor')
        self.write_encode_code(dest_dir)
        tasks = self.encode_includes(src_dir, dest_dir)
        return tasks

    def encode_root(self):
        return [[
            "2",
            self.header["file_uuid"],
            ""
        ]]

    def encode_empty_data(self):
        return {
            "copyinfo": [
                [
                    "4",
                    [
                        "0"
                    ],
                    [
                        "0"
                    ],
                    [
                        "0"
                    ],
                    [
                        "0",
                        "0"
                    ],
                    [
                        "0"
                    ]
                ]
            ]
        }



import argparse
import json
import os
import shutil
import sys
import tempfile
from datetime import datetime

from . import __version__
from . import helper
from .container_reader import extract as container_extract, decompress_and_extract
from .container_writer import build as container_build, compress_and_build
from .decoder import decode, encode
from .file_organizer import FileOrganizer
from .file_organizer_ce import FileOrganizerCE
from .index import update_index
from .json_container_decoder import json_decode, json_encode


def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, version=None, descent=None):
    try:
        begin0 = datetime.now()
        print(f"v8unpack {__version__}")
        print(f"{helper.str_time(begin0)} Начали        ", end='')
        if descent is None:
            helper.clear_dir(os.path.normpath(out_dir_name))
        clear_temp_dir = False
        if temp_dir is None:
            clear_temp_dir = True
            temp_dir = tempfile.mkdtemp()
        helper.clear_dir(os.path.normpath(temp_dir))

        dir_stage0 = os.path.join(temp_dir, 'decode_stage_0')
        dir_stage1 = os.path.join(temp_dir, 'decode_stage_1')
        dir_stage2 = os.path.join(temp_dir, 'decode_stage_2')
        dir_stage3 = os.path.join(temp_dir, 'decode_stage_3')

        if index:
            try:
                with open(index, 'r', encoding='utf-8') as f:
                    index = json.load(f)
            except FileNotFoundError:
                index = None

        pool = helper.get_pool()

        begin1 = datetime.now()
        print(f" - {begin1 - begin0}\n{helper.str_time(begin1)} Распаковываем ", end='')
        container_extract(in_filename, dir_stage0, False, False)
        decompress_and_extract(dir_stage0, dir_stage1, pool=pool)

        begin2 = datetime.now()
        print(f" - {begin2 - begin1}\n{helper.str_time(begin2)} Конвертируем  ", end='')
        json_decode(dir_stage1, dir_stage2, pool=pool)

        begin3 = datetime.now()
        print(f" - {begin3 - begin2}\n{helper.str_time(begin0)} Раcшифровываем", end='')
        decode(dir_stage2, dir_stage3, pool=pool, version=version)

        begin4 = datetime.now()
        print(f" - {begin4 - begin3}\n{helper.str_time(begin0)} Организуем    ", end='')
        if descent:
            FileOrganizerCE.unpack(dir_stage3, out_dir_name, pool=pool, index=index, descent=descent)
        else:
            FileOrganizer.unpack(dir_stage3, out_dir_name, pool=pool, index=index)

        end = datetime.now()
        print(f" - {end - begin4}\n{helper.str_time(end)} Готово         - {end - begin0}")

        helper.close_pool(pool)
        if clear_temp_dir:
            shutil.rmtree(temp_dir, ignore_errors=True)
    except Exception as err:
        print(err)


def build(in_dir_name: str, out_file_name: str, *, temp_dir=None, index=None,
          version='803', descent=None, release=None):
    try:
        begin0 = datetime.now()
        print(f"v8unpack {__version__}")
        print(f"{helper.str_time(begin0)} Начали        ", end='')
        clear_temp_dir = False
        if temp_dir is None:
            clear_temp_dir = True
            temp_dir = tempfile.mkdtemp()
        helper.clear_dir(os.path.normpath(temp_dir))

        dir_stage0 = os.path.join(temp_dir, 'encode_stage_0')
        dir_stage1 = os.path.join(temp_dir, 'encode_stage_1')
        dir_stage2 = os.path.join(temp_dir, 'encode_stage_2')
        dir_stage3 = os.path.join(temp_dir, 'encode_stage_3')

        pool = helper.get_pool()

        if index:
            try:
                with open(index, 'r', encoding='utf-8') as f:
                    index = json.load(f)
            except FileNotFoundError:
                index = None

        begin1 = datetime.now()
        print(f" - {begin1 - begin0}\n{helper.str_time(begin1)} Собираем      ", end='')
        if descent:
            FileOrganizerCE.pack(in_dir_name, dir_stage3, pool=pool, index=index, descent=descent)
        else:
            FileOrganizer.pack(in_dir_name, dir_stage3, pool=pool, index=index)

        begin2 = datetime.now()
        print(f" - {begin2 - begin1}\n{helper.str_time(begin2)} Зашифровываем ", end='')
        encode(dir_stage3, dir_stage2, version=version, pool=pool, release=release,
               file_name=os.path.basename(out_file_name))

        begin3 = datetime.now()
        print(f" - {begin3 - begin2}\n{helper.str_time(begin0)} Конвертируем  ", end='')
        json_encode(dir_stage2, dir_stage1, pool=pool)

        begin4 = datetime.now()
        print(f" - {begin4 - begin3}\n{helper.str_time(begin0)} Запаковываем  ", end='')
        compress_and_build(dir_stage1, dir_stage0, pool=pool)
        container_build(dir_stage0, out_file_name, True)

        end = datetime.now()
        print(f" - {end - begin4}\n{helper.str_time(end)} Готово         - {end - begin0}")

        helper.close_pool(pool)
        if clear_temp_dir:
            shutil.rmtree(temp_dir, ignore_errors=True)
    except Exception as err:
        print(err)


def main():
    parser = argparse.ArgumentParser(
        prog=f'v8unpack {__version__}',
        description='Распаковка и сборка бинарных файлов 1С'
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-E', nargs=2, metavar=('file', 'src'),
                       help='разобрать файл 1С, где '
                            'file - путь до бинарного файла, '
                            'src -  папка куда будут помещены исходники')
    group.add_argument('-B', nargs=2, metavar=('src', 'file'),
                       help='собрать файл 1С, где '
                            'src - путь до папки с исходниками, '
                            'file - путь до бинарного файла')
    group.add_argument('-I', nargs=1, metavar='src',
                       help='сформировать index, где '
                            'src - путь до папки с исходниками'
                       )
    parser.add_argument('--temp', help='путь до временной папки')
    parser.add_argument('--core', help='название общей папки добавляемой в индекс по умолчанию')
    parser.add_argument('--index', help='путь до json файла с словарем копирования,'
                                        'структура файла: {путь исходника: путь общей папки}')
    parser.add_argument('--version', default='803',
                        help="версия сборки, для сборки обработок указывается версия платформы 801/802/803, "
                             " для сборки расширений указывается версия режима совместимости, "
                             "например для 8.3.6 это 80306, подробности в документации на github")
    parser.add_argument('--descent',
                        help="включает режим наследования при сборке и разборке,"
                             "четырех значный формат 3.0.75.100 (не более 3 знаков на каждый разряд)"
                             "подробности в инструкции - раздел разработка расширений")
    parser.add_argument('--release',
                        help="номер версии собираемого продукта, "
                             "для обработки добавяется в функцию GetVersion модуля обработки, "
                             "для расширений устанавливается в соответствующий реквизит ")

    if len(sys.argv) == 1:
        parser.print_help()
        return 1

    args = parser.parse_args()
    descent = int(args.descent) if args.descent else None

    if args.E is not None:
        extract(os.path.abspath(args.E[0]), os.path.abspath(args.E[1]),
                index=args.index, temp_dir=args.temp, version=args.version, descent=descent)

    if args.B is not None:
        build(os.path.abspath(args.B[0]), os.path.abspath(args.B[1]),
              index=args.index, temp_dir=args.temp, version=args.version, descent=descent)

    if args.I is not None:
        update_index(args.I[0], args.index, args.core)


if __name__ == '__main__':
    sys.exit(main())



from enum import Enum


class MetaDataTypes(Enum):
    # todo в синтаксис помощнике найти все английские названия искать строкой "ОбъектМетаданных: [хxx...]"
    ExternalDataProcessor = 'c3831ec8-d8d5-4f93-8a22-f9bfae07327f'    # Внешняя обработка
    Configuration = '9cd510cd-abfc-11d4-9434-004095e12fc7'            # Идентифицирует род. узел описания конфигурации
    CommandGroup = '1c57eabe-7349-44b3-b1de-ebfeab67b47d'             # Группа команд
    TabularSectionAttribute = '2bcef0d1-0981-11d6-b9b8-0050bae0a95d'  # Атрибут Табличной части Внешней обработки
    FormAttribute = 'ec6bb5e5-b7a8-4d75-bec9-658107a699cf'            # Атрибут Внешней обработки
    Document = "061d872a-5787-460e-95ac-ed74ea3a3e84"                 # Документы
    # DocumentAttribute = "45e46cbc-3e24-4165-8b7b-cc98a6f80211"        # Документ.Реквизиты
    DocumentCommand = 'b544fc6a-2ba3-4885-8fb2-cb289fb6d65e'          # Команда документа
    DocumentForm = "fb880e93-47d7-4127-9357-a20e69c17545"             # Форма документа
    DataProcessor = "bf845118-327b-4682-b5c6-285d2a0eb296"            # Обработки
    Template = '3daea016-69b7-4ed4-9453-127911372fe6'                 # Макет внешней обработки
    Form = 'd5b0e5ed-256d-401c-9c36-f630cafd8a62'                     # Форма внешней обработки
    Catalog = "cf4abea6-37b2-11d4-940f-008048da11f9"                  # Справочники
    CatalogForm = 'fdf816d2-1ead-11d5-b975-0050bae0a95d'              # Форма элемента справочника
    CatalogCommand = '4fe87c89-9ad4-43f6-9fdb-9dc83b3879c6'           # Команда справочника
    CommonPicture = '7dcd43d9-aca5-4926-b549-1842e6a4e8cf'            # Общие картинки
    CommonCommand = '2f1a5187-fb0e-4b05-9489-dc5dd6412348'            # Общая команда
    CommonTemplate = '0c89c792-16c3-11d5-b96b-0050bae0a95d'           # Общие макеты
    CommonModule = '0fe48980-252d-11d6-a3c7-0050bae0a776'             # Общие модули
    CommonForm = "07ee8426-87f1-11d5-b99c-0050bae0a95d"               # Общие формы
    Report = '631b75a0-29e2-11d6-a3c7-0050bae0a776'                   # Отчеты
    InformationRegister = '13134201-f60b-11d5-a3c7-0050bae0a776'      # Регистры сведений
    InformationRegisterForm = '13134204-f60b-11d5-a3c7-0050bae0a776'      # Form регистра сведений
    InformationRegisterCommand = 'b44ba719-945c-445c-8aab-1088fa4df16e'
    Subsystem = '37f2fa9a-b276-11d4-9435-004095e12fc7'                    # Подсистемы
    Role = '09736b02-9cac-4e3f-b4f7-d3e9576ab948'                         # Роли
    Language = '9cd510ce-abfc-11d4-9434-004095e12fc7'                     # Языки
    SessionParameter = '24c43748-c938-45d0-8d14-01424a72b11e'             # Параметры сеанса
    Enum = "f6a80749-5ad7-400b-8519-39dc5dff2542"                         # Перечисления
    ChartOfCharacteristicType = '82a1b659-b220-4d94-a9bd-14d757b95a48'    # План видов характеристик
    # ChartOfCalculationTypes = '30b100d6-b29f-47ac-aec7-cb8ca8a54767'    # План видов расчета
    # ExchangePlan = '857c4a91-e5f4-4fac-86ec-787626f1c108'               # План обмена
    # ChartOfAccounts = '238e7e88-3c5f-48b2-8a3b-81ebbecb20ed'            # План счетов
    # AccumulationRegister = 'b64d9a40-1642-11d6-a3c7-0050bae0a776'       # Регистры накопления
    # CalculationRegister = 'f2de87a8-64e5-45eb-a22d-b3aedab050e7'        # Регистры расчета
    # Style = '3e5404af-6ef8-4c73-ad11-91bd2dfac4c8'                      # Стиль
    # AccountingRegister = '2deed9b8-0056-4ffe-a473-c20a6c32a0bc'         # Регистры бухгалтерии
    StyleItem = '58848766-36ea-4076-8800-e91eb49590d7'                    # Элемены стиля
    # HTTPService = '0fffc09c-8f4c-47cc-b41c-8d5c5a221d79'                # HTTPСервис
    # XDTOPackage = 'cc9df798-7c94-4616-97d2-7aa0b7bc515e'                # ПакетXDTO
    # WebService = '8657032e-7740-4e1d-a3ba-5dd6e8afb78f'                 # Веб-сервис
    # CommonAttribute = '15794563-ccec-41f6-a83c-ec5f7b9a5bc1'            # Общие реквизиты
    # FilterCriterion = '3e7bfcc0-067d-11d6-a3c7-0050bae0a776'            # Критерии отбора
    # EventSubscription = '4e828da6-0f44-4b5b-b1c0-a2b3cfe7bdcc'          # Подписки на события
    # ScheduledJob = '11bdaf85-d5ad-4d91-bb24-aa0eee139052'               # Регламентные задания
    FunctionalOption = 'af547940-3268-434f-a3e7-e47d6d2638c3'             # Функциональные опции
    # FunctionalOptionsParameter = '30d554db-541e-4f62-8970-a1c6dcfeb2bc' # Параметры функциональных опций
    DefinedType = 'c045099e-13b9-4fb6-9d50-fca00202971e'                  # Определяемые типы
    # SettingsStorage = '46b4cd97-fd13-4eaa-aba2-3bddd7699218'            # Хранилище настроек
    # Interface = '39bddf6a-0c3c-452b-921c-d99cfa1c2f1b'                  # Интерфейсы
    # WSReference = 'd26096fb-7a5d-4df9-af63-47d04771fa9b'                # WS-ссылки
    Constant = '0195e80c-b157-11d4-9435-004095e12fc7'                     # Константы
    # DocumentJournal = '5aee69df-0513-4c6c-9815-103102471712'            # Журналы документов
    # BusinessProcess = 'fcd3404e-1523-48ce-9bc0-ecdb822684a1'            # Бизнес-процессы
    # Task = '3e63355c-1378-4953-be9b-1deb5fb6bec5'                       # Задачи
    # ExternalDataSource = 'e68182ea-4237-4383-967f-90c1e3370bc7'         # Внешние источники данных (не точно...)






from ..MetaDataObject.core.Container import Container


class DataProcessor(Container):
    version = '803'
    ext_code = {
        'mgr': '2',  # модуль менеджера
        'obj': '0',  # модуль объекта
    }

    def __init__(self):
        super(DataProcessor, self).__init__()

    @classmethod
    def get_decode_header(cls, header_data):
        return header_data[0][1][3][1]



from ..MetaDataObject.core.Simple import SimpleNameFolder
from ..MetaDataObject.versions.Form801 import Form801
from ..MetaDataObject.versions.Form802 import Form802
from ..MetaDataObject.versions.Form803 import Form803


class Form(SimpleNameFolder):
    versions = {
        '801': Form801,
        '802': Form802,
        '803': Form803
    }



from .Template8x import Template8x
from ... import helper


class Template803(Template8x):
    def encode_header(self):
        return [[
            "1",
            [
                "2",
                self.tmpl_type.value,
                [
                    self.header['h0'],
                    [
                        self.header['h1_0'],
                        "0",
                        self.header['uuid']
                    ],
                    helper.str_encode(self.header['name']),
                    helper.encode_name2(self.header),
                    helper.str_encode(self.header['comment']),
                    *self.header['h5']
                ]
            ],
            "0"
        ]]



import re

from .Form8x import Form8x
from ... import helper
from ...ext_exception import ExtException

OLD_FORM = '0'
UPR_FORM = '1'


class Form803(Form8x):
    ver = '803'
    double_quotes = re.compile('("")')
    quotes = re.compile('(")')

    def decode_data(self, src_dir, uuid):
        _header_obj = self.get_decode_obj_header(self.header['data'])
        self.header['Включать в содержание справки'] = _header_obj[1][2]
        self.header['Тип формы'] = _header_obj[1][3]
        self.header['Версия803'] = _header_obj[1][0]

        try:
            self.header['Расширенное представление'] = _header_obj[2]
        except IndexError:
            pass

        if self.header['Тип формы'] != OLD_FORM:
            self.decode_form0(src_dir, uuid)

    def decode_form0(self, src_dir, uuid):
        try:
            form = helper.json_read(src_dir, f'{uuid}.0.json')
        except FileNotFoundError:
            return
        try:
            _code = helper.str_decode(self.getset_form_code(form, 'Код в отдельном файле', self.header))
            if _code:
                _code = self.double_quotes.sub('"', _code)
                self.code['obj'] = _code
                self.header['code_info_obj'] = 'Код в отдельном файле'
        except Exception as err:
            raise ExtException(parent=err, detail=self.header['uuid'])
        self.form.append(form)

    @classmethod
    def get_last_level_array(cls, data):
        while True:
            if isinstance(data[-1], list):
                return cls.get_last_level_array(data[-1])
            else:
                return data

    @classmethod
    def getset_form_code(cls, form, new_value=None, header=None):
        err_detail = f'{header["uuid"]} {header["name"]} ' \
                     f'опытным путем подобрано, если у Вас код не где то не достается' \
                     f'обновитесь до последней версии, и если не поможет создайте issue с дампом'
        len_form_0 = len(form[0])
        if len_form_0 > 2 and form[0][0] in ['4', '3']:
            code = form[0][2]
            if not isinstance(code, str):
                raise ExtException(
                    message='Not supported forms',
                    detail=err_detail,
                    dump=form
                )
            form[0][2] = new_value
            if len_form_0 != 10:
                a = 1
            return code

        last_level = cls.get_last_level_array(form)
        if len_form_0 < 10 and (last_level[0] == '49' or last_level[0] == '4'):
            return ''

        if len(last_level) > 10 \
                and last_level[0] in ['22', '1'] \
                and last_level[-1] == '0' \
                and last_level[-2] == '0':
            code_index = -8
            code = last_level[code_index]
            if not isinstance(code, str):
                raise ExtException(
                    message='Not supported forms',
                    detail=err_detail,
                    dump=form
                )
            if new_value is not None:
                last_level[code_index] = new_value
            return code
        return ''

    def decode_form1(self, src_dir, uuid):
        try:
            form = helper.json_read(src_dir, f'{uuid}.1.json')
        except FileNotFoundError:
            return
        self.form.append(form)

    def decode_code(self, src_dir):
        if self.header['Тип формы'] == OLD_FORM:
            self.decode_old_form(src_dir)
        else:
            super().decode_code(src_dir)
        self.decode_form1(src_dir, self.header['uuid'])

    def encode_header(self):
        return [[
            "1",
            [
                "1",
                [
                    "0",
                    self.encode_header_title(),
                    self.header.get('Расширенное представление', ['0'])
                ]
            ],
            "0"
        ]]

    def encode_header_title(self):
        return [
            self.header.get('Версия803', '13'),
            [
                self.header['h0'],
                [
                    self.header['h1_0'],
                    "0",
                    self.header['uuid']
                ],
                helper.str_encode(self.header['name']),
                helper.encode_name2(self.header),
                helper.str_encode(self.header['comment']),
                *self.header['h5'],
            ],
            self.header['Включать в содержание справки'],
            self.header['Тип формы'],
            [
                "2",
                [
                    "\"#\"",
                    "1708fdaa-cbce-4289-b373-07a5a74bee91",
                    "1"
                ],
                [
                    "\"#\"",
                    "1708fdaa-cbce-4289-b373-07a5a74bee91",
                    "2"
                ]
            ]
        ]

    def encode_data(self):
        if self.header['Тип формы'] == OLD_FORM:
            return
        try:
            _code = self.code.pop('obj', "")
            _code = self.quotes.sub('""', _code)
            form0 = self.form[0]
            self.getset_form_code(form0, helper.str_encode(_code), self.header)
        except IndexError:
            pass
        return self.form

    def write_encode_object(self, dest_dir):
        if self.header['Тип формы'] == OLD_FORM:
            self.write_old_encode_object(dest_dir)
        else:
            helper.json_write(self.encode_header(), dest_dir, f'{self.header["uuid"]}.json')
            if self.form:
                helper.json_write(self.form[0], dest_dir, f'{self.header["uuid"]}.0.json')
        if self.form and len(self.form) > 1:
            helper.json_write(self.form[1], dest_dir, f'{self.header["uuid"]}.1.json')

    def encode_empty_form(self):
        return [[
            "4",
            [
                "49",
                "0",
                "0",
                "0",
                "0",
                "1",
                "0",
                "0",
                "00000000-0000-0000-0000-000000000000",
                "1",
                [
                    "1",
                    "0"
                ],
                "0",
                "0",
                "1",
                "1",
                "1",
                "0",
                "1",
                "0",
                [
                    "0",
                    "1",
                    "0"
                ],
                [
                    "0"
                ],
                "1",
                [
                    "22",
                    [
                        "-1",
                        "02023637-7868-4a5f-8576-835a76e0c9ba"
                    ],
                    "0",
                    "0",
                    "0",
                    "9",
                    "\"\"",
                    [
                        "1",
                        "0"
                    ],
                    [
                        "1",
                        "0"
                    ],
                    "0",
                    "1",
                    "0",
                    "0",
                    "0",
                    "2",
                    "2",
                    [
                        "3",
                        "4",
                        [
                            "0"
                        ]
                    ],
                    [
                        "7",
                        "3",
                        "0",
                        "1",
                        "100"
                    ],
                    [
                        "0",
                        "0",
                        "0"
                    ],
                    "1",
                    [
                        "0",
                        "0",
                        "1"
                    ],
                    "0",
                    "1",
                    "0",
                    "0",
                    "0",
                    "3",
                    "3",
                    "0"
                ],
                "0",
                "\"\"",
                "\"\"",
                "1",
                [
                    "22",
                    [
                        "0"
                    ],
                    "0",
                    "0",
                    "0",
                    "7",
                    "\"Navigator\"",
                    [
                        "1",
                        "0"
                    ],
                    [
                        "1",
                        "0"
                    ],
                    "0",
                    "1",
                    "0",
                    "0",
                    "0",
                    "2",
                    "2",
                    [
                        "3",
                        "4",
                        [
                            "0"
                        ]
                    ],
                    [
                        "7",
                        "3",
                        "0",
                        "1",
                        "100"
                    ],
                    [
                        "0",
                        "0",
                        "0"
                    ],
                    "0",
                    "0",
                    "1",
                    "0",
                    "1",
                    [
                        "11",
                        [
                            "0"
                        ],
                        "0",
                        "0",
                        "0",
                        "0",
                        "\"NavigatorExtendedTooltip\"",
                        [
                            "1",
                            "0"
                        ],
                        [
                            "1",
                            "0"
                        ],
                        "1",
                        "0",
                        "0",
                        "2",
                        "2",
                        [
                            "3",
                            "4",
                            [
                                "0"
                            ]
                        ],
                        [
                            "7",
                            "3",
                            "0",
                            "1",
                            "100"
                        ],
                        [
                            "0",
                            "0",
                            "0"
                        ],
                        "1",
                        [
                            "5",
                            "0",
                            "0",
                            "3",
                            "0",
                            [
                                "0",
                                "1",
                                "0"
                            ],
                            [
                                "3",
                                "4",
                                [
                                    "0"
                                ]
                            ],
                            [
                                "3",
                                "4",
                                [
                                    "0"
                                ]
                            ],
                            [
                                "3",
                                "0",
                                [
                                    "0"
                                ],
                                "0",
                                "1",
                                "0",
                                "48312c09-257f-4b29-b280-284dd89efc1e"
                            ]
                        ],
                        "0",
                        "1",
                        "2",
                        [
                            "1",
                            [
                                "1",
                                "0"
                            ],
                            "0"
                        ],
                        "0",
                        "0",
                        "1",
                        "0",
                        "0",
                        "1",
                        "0",
                        "3",
                        "3",
                        "0"
                    ],
                    "0",
                    "3",
                    "3",
                    "0"
                ],
                "1",
                "\"\"",
                "0",
                "0",
                "0",
                "0",
                "0",
                "0",
                "3",
                "3",
                "0",
                "0",
                "0",
                "100",
                "1",
                "1",
                "0",
                "0",
                "0",
                [
                    "49",
                    "0"
                ]
            ],
            "Код в отдельном файле",
            [
                "4",
                "1",
                [
                    "9",
                    [
                        "1"
                    ],
                    "0",
                    "\"Объект\"",
                    [
                        "1",
                        "0"
                    ],
                    [
                        "\"Pattern\"",
                        [
                            "\"#\"",
                            "7cadabb8-6ade-4026-bc22-5e7e789b0b4b"
                        ]
                    ],
                    [
                        "0",
                        [
                            "0",
                            [
                                "\"B\"",
                                "1"
                            ],
                            "0"
                        ]
                    ],
                    [
                        "0",
                        [
                            "0",
                            [
                                "\"B\"",
                                "1"
                            ],
                            "0"
                        ]
                    ],
                    [
                        "0",
                        "0"
                    ],
                    [
                        "0",
                        "0"
                    ],
                    "1",
                    "0",
                    "0",
                    "0",
                    [
                        "0",
                        "0"
                    ],
                    [
                        "0",
                        "0"
                    ]
                ],
                "0",
                "0",
                [
                    "#base64:77u/PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiPz4NCjxTZXR0aW5ncyB4bWxucz0iaHR0cDovL3Y4LjFjLnJ1LzguMS9kYXRhLWNvbXBvc2l0aW9uLXN5c3RlbS9zZXR0aW5ncyIgeG1sbnM6ZGNzY29yPSJodHRwOi8vdjguMWMucnUvOC4xL2RhdGEtY29tcG9zaXRpb24tc3lzdGVtL2NvcmUiIHhtbG5zOnN0eWxlPSJodHRwOi8vdjguMWMucnUvOC4xL2RhdGEvdWkvc3R5bGUiIHhtbG5zOnN5cz0iaHR0cDovL3Y4LjFjLnJ1LzguMS9kYXRhL3VpL2ZvbnRzL3N5c3RlbSIgeG1sbnM6djg9Imh0dHA6Ly92OC4xYy5ydS84LjEvZGF0YS9jb3JlIiB4bWxuczp2OHVpPSJodHRwOi8vdjguMWMucnUvOC4xL2RhdGEvdWkiIHhtbG5zOndlYj0iaHR0cDovL3Y4LjFjLnJ1LzguMS9kYXRhL3VpL2NvbG9ycy93ZWIiIHhtbG5zOndpbj0iaHR0cDovL3Y4LjFjLnJ1LzguMS9kYXRhL3VpL2NvbG9ycy93aW5kb3dzIiB4bWxuczp4cz0iaHR0cDovL3d3dy53My5vcmcvMjAwMS9YTUxTY2hlbWEiIHhtbG5zOnhzaT0iaHR0cDovL3d3dy53My5vcmcvMjAwMS9YTUxTY2hlbWEtaW5zdGFuY2UiPg0KCTxvdXRwdXRQYXJhbWV0ZXJzLz4NCjwvU2V0dGluZ3M+"
                ]
            ],
            [
                "0",
                "0"
            ],
            [
                "0",
                "0"
            ],
            [
                "0",
                "0"
            ],
            [
                "0",
                "0"
            ],
            "0",
            "0"
        ]]



from .Form803 import Form803


class DocumentForm803(Form803):
    @classmethod
    def get_decode_obj_header(cls, header):
        return header[0][1]

    def encode_header(self):
        return [[
            "1",
            [
                "0",
                self.encode_header_title(),
            ],
            "0"
        ]]



# -*- coding: utf-8 -*-
import collections
import datetime
import io
import os
import zlib
from struct import pack, unpack, calcsize

from . import helper

# INT32_MAX 7fffffff
END_MARKER = 2147483647
# INT64_MAX ffffffffffffffff для конфигураций старше 8.3.15
END_MARKER_64 = 18446744073709551615
Header = collections.namedtuple('Header', 'first_empty_block_offset, default_block_size')
Block = collections.namedtuple('Block', 'doc_size, current_block_size, next_block_offset, data')
Document = collections.namedtuple('Document', 'size, data')
File = collections.namedtuple('File', 'name, size, created, modified, data')


def read_header(file):
    """
    Считывыет заголовок контейнера.

    :param file: объект файла контейнера
    :type file: BufferedReader
    :return: Заголовок контейнера
    :rtype: Header
    """
    file.seek(0)
    fmt = '4i'
    buff = file.read(calcsize(fmt))
    header = unpack(fmt, buff)
    return Header(header[0] if header[0] != END_MARKER else None, header[1])


def read_block(file, offset, max_data_length=None):
    """
    Считывает блок данных из контейнера.

    :param file: объект файла контейнера
    :type file: BufferedReader
    :param offset: смещение блока в файле контейнера (байт)
    :type offset: int
    :param max_data_length: максимальный размер считываемых данных из блока (байт)
    :type max_data_length: int
    :return: объект блока данных
    :rtype: Block
    """
    file.seek(offset)
    fmt = '2s8s1s8s1s8s1s2s'
    buff = file.read(calcsize(fmt))
    if not buff:
        return
    header = unpack(fmt, buff)

    doc_size = int(header[1], 16)
    current_block_size = int(header[3], 16)
    next_block_offset = int(header[5], 16)

    if max_data_length is None:
        max_data_length = min(current_block_size, doc_size)

    data = file.read(min(current_block_size, max_data_length))

    return Block(doc_size, current_block_size, next_block_offset, data)


def read_document_gen(file, offset):
    """
    Создает генератор чтения данных документа в контейнере.
    Первое значение генератора - размер документа (байт).
    Остальные значения - данные блоков, составляющих документ

    :param file: объект файла контейнера
    :type file: BufferedReader
    :param offset: смещение документа в контейнере (байт)
    :type offset: int
    :return: генератор чтения данных документа
    """
    file.seek(offset)
    header_block = read_block(file, offset)
    if header_block is None:
        return
    else:
        yield header_block.doc_size
        yield header_block.data

        left_bytes = header_block.doc_size - len(header_block.data)
        next_block_offset = header_block.next_block_offset

        while left_bytes > 0 and next_block_offset != END_MARKER:
            block = read_block(file, next_block_offset, left_bytes)
            left_bytes -= len(block.data)
            yield block.data
            next_block_offset = block.next_block_offset


def read_document(file, offset):
    """
    Считывает документ из контейнера. В качестве данных документа возвращается генератор.

    :param file: объект файла контейнера
    :type file: BufferedReader
    :param offset: смещение документа в контейнере
    :type offset: int
    :return: объект документа
    :rtype: Document
    """
    gen = read_document_gen(file, offset)

    try:
        size = next(gen)
    except StopIteration:
        size = 0
    return Document(size, gen)


def read_full_document(file, offset):
    """
    Считывает документ из контейнера. Данные документа считываются целиком.

    :param file: объект файла контейнера
    :type file: BufferedReader
    :param offset: смещение документа в контейнере (байт)
    :type offset: int
    :return: объект документа
    :rtype: Document
    """
    document = read_document(file, offset)
    return Document(document.size, b''.join([chunk for chunk in document.data]))


def parse_datetime(time):
    """
    Преобразует внутренний формат хранения дат файлов в контейнере в обычную дату

    :param time: внутреннее представление даты
    :type time: string
    :return: дата/время
    :rtype: datetime
    """
    # TODO проверить работу на *nix, т.к там начало эпохи - другая дата
    return datetime.datetime(1, 1, 1) + datetime.timedelta(microseconds=time * 100)


def read_entries(file):
    """
    Считывает оглавление контейнера

    :param file: объект файла контейнера
    :type file: BufferedReader
    :return: словарь файлов в контейнере
    :rtype: OrderedDict
    """
    # Первый документ после заголовка содержит оглавление
    doc = read_full_document(file, calcsize('4i'))
    table_of_contents = [unpack('2i', x) for x in doc.data.split(pack('i', END_MARKER))[:-1]]

    files = collections.OrderedDict()
    for file_description_offset, file_data_offset in table_of_contents:
        file_description_document = read_full_document(file, file_description_offset)
        file_data = read_document(file, file_data_offset)

        fmt = ''.join(['QQi', str(file_description_document.size - calcsize('QQi')), 's'])
        file_description = unpack(fmt, file_description_document.data)

        # Из описания формата длина имени файла определяется точно, поэтому, теоретически, мусора быть не должно
        # По факту имя часто имеет в конце мусор, который чаще всего состоит из последовательности \x00 n-раз,
        # но иногда бывают и другие символы после \x00. Поэтому применяем вот такой костыль:
        name = file_description[3].decode('utf-16').partition('\x00')[0]
        inner_file = File(name, file_data.size, parse_datetime(file_description[0]),
                          parse_datetime(file_description[1]), file_data.data)

        files[inner_file.name] = inner_file

    return files


class ContainerReader(object):
    """
    Класс для чтения контейнеров
    """

    def __init__(self, file):
        header = read_header(file)
        if header.default_block_size == 0:
            raise BufferError('Container is empty')

        self.file = file
        self.first_empty_block_offset = header.first_empty_block_offset
        self.default_block_size = header.default_block_size
        #: Список файлов в контейнере
        self.entries = read_entries(self.file)

    def extract(self, path, deflate=False, recursive=False):
        """
        Распаковывает содержимое контейнера в каталог

        :param path: каталог распаковки
        :type path: string
        :param deflate: разархивировать содержимое файлов
        :type deflate: bool
        :param recursive: выполнять рекурсивно
        :type recursive: bool
        """
        helper.clear_dir(path)
        if not self.entries:
            print('Пустой контейнер = распаковывать нечего')
            return

        for filename, file_obj in self.entries.items():
            self.extract_file(filename, file_obj, path, deflate, recursive)

    @staticmethod
    def extract_file(filename, file_obj, path, deflate=False, recursive=False):
        file_path = os.path.join(path, filename)
        with open(file_path, 'wb') as f:
            if deflate:
                # wbits = -15 т.к. у архивированных файлов нет заголовков
                decompressor = zlib.decompressobj(-15)
                for chunk in file_obj.data:
                    decomressed_chunk = decompressor.decompress(chunk)
                    f.write(decomressed_chunk)
            else:
                for chunk in file_obj.data:
                    f.write(chunk)

        if not recursive:
            return

        # Каждый файл внутри контейнера может быть контейнером
        # Для проверки является ли файл контейнером проверим первые 4 бита
        # Способ проверки ненадежный - нужно придумать что-то другое
        file_is_container = False
        with open(file_path, 'rb') as f:
            if f.read(4) == b'\xFF\xFF\xFF\x7F':
                file_is_container = True
        if file_is_container:
            temp_name = file_path + '.tmp'
            os.rename(file_path, temp_name)
            with open(temp_name, 'rb') as f:
                ContainerReader(f).extract(file_path, recursive=True)
            os.remove(temp_name)


def extract(filename, folder, deflate=True, recursive=True):
    """
    Распаковка контейнера. Сахар для ContainerReader

    :param filename: полное имя файла-контейнера
    :type filename: string
    :param folder: каталог назначения
    :type folder: string
    :param deflate: паспаковка
    :type deflate: boolean
    :param recursive: рекурсивно достаем все контейнеры
    :type recursive: boolean
    """
    with open(filename, 'rb') as f:
        ContainerReader(f).extract(folder, deflate, recursive)


def decompress_and_extract(src_folder, dest_folder, *, pool=None):
    helper.clear_dir(dest_folder)
    entries = os.listdir(src_folder)

    for filename in entries:
        src_path = os.path.join(src_folder, filename)
        dest_path = os.path.join(dest_folder, filename)
        with open(src_path, 'rb') as f:
            # wbits = -15 т.к. у архивированных файлов нет заголовков
            decompressor = zlib.decompressobj(-15)
            data = decompressor.decompress(f.read())

        # Каждый файл внутри контейнера может быть контейнером
        # Для проверки является ли файл контейнером проверим первые 4 бита
        # Способ проверки ненадежный - нужно придумать что-то другое
        file_is_container = data[0:4] == b'\xFF\xFF\xFF\x7F'

        if file_is_container:
            ContainerReader(io.BytesIO(data)).extract(dest_path, recursive=True)
        else:
            with open(dest_path, 'wb') as f:
                f.write(data)



import setuptools
from src.v8unpack import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='v8unpack',
    version=__version__,
    test_requires=[],
    url='https://github.com/saby-integration/v8unpack',
    license='MIT',
    author='Razgovorov Mikhail',
    author_email='1338833@gmail.com',
    description='Unpacking binaries 1C to JSON for GIT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Natural Language :: Russian',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        "Operating System :: OS Independent"
    ],
    keywords='1C CF CFE EPF V8UNPACK SABY SBIS СБИС 1С',
    python_requires='>=3.3',
    zip_safe=False,
    install_requires=[
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    entry_points={
        'console_scripts':
            [
                'v8unpack = v8unpack:main',
            ]
    }
)



from ...MetaDataObject import MetaDataObject


class Simple(MetaDataObject):

    def decode_object(self, src_dir, file_name, dest_dir, dest_path, version, header_data):
        super().decode_object(src_dir, file_name, dest_dir, dest_path, version, header_data)
        self.decode_code(src_dir)

    def decode_includes(self, src_dir, dest_dir, dest_path, header):
        return []

    def encode_includes(self, src_dir, dest_dir):
        return []

    def encode_object(self, src_dir, file_name, dest_dir, version):
        self.encode_code(src_dir, file_name)
        return []


class SimpleNameFolder(Simple):
    def set_write_decode_mode(self, dest_dir, dest_path):
        self.set_mode_decode_in_name_folder(dest_dir, dest_path)

    @classmethod
    def encode_get_include_obj(cls, src_dir, dest_dir, include, tasks, version):
        cls.encode_get_include_obj_from_named_folder(src_dir, dest_dir, include, tasks, version)

    @classmethod
    def get_encode_file_name(cls, file_name):
        return cls.get_obj_name()
